from __future__ import absolute_import

import sys
import modulemd
import logging
import dnf
from . import _depchase, _repodata

def _name_only(rpm_name):
    name, version, release = rpm_name.rsplit("-", 2)
    return name
    
class ModuleRepoquery(object):
    
    def list_modules(self):
        _repodata._populate_module_reverse_lookup()
        module_names = _repodata.list_modules()
        if module_names:
            for name in module_names:
                print(name)

    def list_modules_for_rpm(self, pkg):
        _repodata._populate_module_reverse_lookup()
        module_names = _repodata.get_modules_for_rpm(pkg)
        if module_names:
            for name in module_names:
                print(name)

    def list_rpms_in_module(self, module, full_nevra=False):
        _repodata._populate_module_reverse_lookup()
        rpm_names = _repodata.get_rpms_in_module(module)
        if rpm_names:
            for name in rpm_names:
                if full_nevra:
                    print(name)
                else:
                    print(_name_only(name))
    
    def list_pkg_deps(self, pkgs, module_deps):
        _repodata._populate_module_reverse_lookup()
        pkgs_in_modules = set()
        if module_deps:
            for module in module_deps:
                rpm_names = _repodata.get_rpms_in_module(module)
                pkgs_in_modules |= set(map(lambda x: _name_only(x), rpm_names))
                
        pool = _depchase.make_pool("x86_64")
        run_deps = _depchase.ensure_installable(pool, pkgs)
        rpm_names = run_deps - pkgs_in_modules
        if rpm_names:
            for name in rpm_names:
                print(name)

    def list_modularized_pkgs(self, duplicate_only=False, list_modules=False):
        _repodata._populate_module_reverse_lookup()
        rpm_names = _repodata.get_rpm_reverse_lookup()
        if rpm_names:
            for name in rpm_names.keys():
                if not (duplicate_only and not len(rpm_names[name]) > 1):
                    if list_modules:
                        print(name + "      (" + ", ".join(rpm_names[name]) + ")")
                    else:
                        print(name)


        