import pytest
import os.path
from _fedmod.cli import ModtoolsCLI
from _fedmod.module_repoquery import ModuleRepoquery

class TestListingModules(object):

    def setup(self):
        self.mr = ModuleRepoquery()
    
    def test_list_modules(self, capfd):
        self.mr.list_modules()
        out, err = capfd.readouterr()

        assert "platform" in out
        assert "host" in out


class TestListingPackages(object):

    def setup(self):
        self.mr = ModuleRepoquery()

    def test_list_packages(self, capfd):
        self.mr.list_modularized_pkgs()
        out, err = capfd.readouterr()

        assert "kernel" in out
        assert "gcc" in out
        assert "(platform)" not in out
    
    def test_list_packages_with_modules(self, capfd):
        self.mr.list_modularized_pkgs(list_modules=True)
        out, err = capfd.readouterr()

        assert "kernel" in out
        assert "gcc" in out
        assert "(platform)" in out


class TestResolvingDependencies(object):

    def setup(self):
        self.mr = ModuleRepoquery()

    def test_list_pkg_deps(self, capfd):
        self.mr.list_pkg_deps(["nginx"], [])
        out, err = capfd.readouterr()

        assert "gzip" in out
        assert "nginx" in out

    def test_list_pkg_deps_with_module_deps(self, capfd):
        self.mr.list_pkg_deps(["nginx"], ["platform", "host"])
        out, err = capfd.readouterr()

        assert "gzip" not in out
        assert "nginx" in out

class TestListingPackagesInModule(object):

    def setup(self):
        self.mr = ModuleRepoquery()

    def test_list_rpms_in_module(self, capfd):
        self.mr.list_rpms_in_module("host")
        out, err = capfd.readouterr()

        assert "kernel" in out
        assert "httpd" not in out

class TestListingModulesWithPackage(object):

    def setup(self):
        self.mr = ModuleRepoquery()

    def test_list_modules_for_rpm(self, capfd):
        self.mr.list_modules_for_rpm("kernel")
        out, err = capfd.readouterr()

        assert "host" in out
        assert "httpd" not in out
